﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.EntityFrameworkCore.Diagnostics;
using Model;
using Model.Interfaces;
using System.Configuration;

namespace MyERP2.DBContext
{
    public class MyERP2Context : DbContext
    {
        public MyERP2Context() {}
        public MyERP2Context(string sqlConnectionString) : this(new DbContextOptionsBuilder<MyERP2Context>().UseSqlServer(sqlConnectionString).Options){}

        internal MyERP2Context(DbContextOptions options) : base(options) { }

        //Defining the Entities:
        public virtual DbSet<Model.MyERP2Model.Adress> Adresses { get; set; }
        public virtual DbSet<Model.MyERP2Model.Article> Articles { get; set; }
        public virtual DbSet<Model.MyERP2Model.ArticleSalesPrice> ArticleSalesPrices { get; set; }
        public virtual DbSet<Model.MyERP2Model.Client> Clients { get; set; }
        public virtual DbSet<Model.MyERP2Model.ContactBase> ContactBases { get; set; }
        public virtual DbSet<Model.MyERP2Model.Currency> Currencies { get; set; }
        public virtual DbSet<Model.MyERP2Model.DiscountGroup> DiscountGroups { get; set; }
        public virtual DbSet<Model.MyERP2Model.FilterDefinition> FilterDefinitions { get; set; }
        public virtual DbSet<Model.MyERP2Model.FilterItem> FilterItems { get; set; }
        public virtual DbSet<Model.MyERP2Model.Incoterm> Incoterms { get; set; }
        public virtual DbSet<Model.MyERP2Model.Language> Languages { get; set; }
        public virtual DbSet<Model.MyERP2Model.LegalForm> LegalForms { get; set; }
        public virtual DbSet<Model.MyERP2Model.NumberRange> NumberRanges { get; set; }
        public virtual DbSet<Model.MyERP2Model.ObjectDescription> ObjectDescriptions { get; set; }
        public virtual DbSet<Model.MyERP2Model.ObjectNote> ObjectNotes { get; set; }
        public virtual DbSet<Model.MyERP2Model.Prospect> Prospects { get; set; }
        public virtual DbSet<Model.MyERP2Model.Protocol> Protocols { get; set; }
        public virtual DbSet<Model.MyERP2Model.Supplier> Suppliers { get; set; }
        public virtual DbSet<Model.MyERP2Model.TextDescription> TextDescriptions { get; set; }
        public virtual DbSet<Model.MyERP2Model.TextDescriptionHeader> TextDescriptionHeaders { get; set; }
        public virtual DbSet<Model.MyERP2Model.Unit> Units { get; set; }
        public virtual DbSet<Model.MyERP2Model.User> Users { get; set; }
        public virtual DbSet<Model.MyERP2Model.UserGroup> UserGroups { get; set; }
        public virtual DbSet<Model.MyERP2Model.UserGroupRight> UserGroupRights { get; set; }
        public virtual DbSet<Model.MyERP2Model.UserSettingsGrid> UserSettingsGrids { get; set; }
        public virtual DbSet<Model.MyERP2Model.UserSettingsGridColumn> UserSettingsGridColumns { get; set; }
        public virtual DbSet<Model.MyERP2Model.UserUserGroup> UserUserGroups { get; set; }
        public virtual DbSet<Model.MyERP2Model.VatGroup> VatGroups { get; set; }

        //Defining the Model

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            if (!optionsBuilder.IsConfigured)
            {
                //Here we have to define a specific server based on the selection during login
                //The connectionstring is stored in the app.config file:
                
                optionsBuilder.UseSqlServer(ConfigurationManager.ConnectionStrings["DatabaseConnection"].ConnectionString).ConfigureWarnings(warnings => warnings.Throw(CoreEventId.IncludeIgnoredWarning));
                //optionsBuilder.UseSqlServer(@"Server=(localdb)\mssqllocaldb;Database=MyERP2;Trusted_Connection=True;").ConfigureWarnings(warnings => warnings.Throw(CoreEventId.IncludeIgnoredWarning));
            }
        }


        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            //Define the model itself
                //Entity Articles:
                modelBuilder.Entity<Model.MyERP2Model.Article>().HasMany(e => e.NvArticleSalesPrices).WithOne(e => e.NvArtikel).HasForeignKey(e => e.ArtNr);
                modelBuilder.Entity<Model.MyERP2Model.Article>().HasMany(e => e.NvNotes).WithOne(e => e.NvArticle).HasForeignKey(e => e.ArtId);
                modelBuilder.Entity<Model.MyERP2Model.Article>().HasMany(e => e.NvDescription).WithOne(e => e.NvArticle).HasForeignKey(e => e.ArtId);

                //Entity ContactBase:
                modelBuilder.Entity<Model.MyERP2Model.ContactBase>().HasMany(e => e.NvAdresses).WithOne(e => e.NvContactBase).HasForeignKey(e => e.ContactBaseId);

                //Entity Currency:
                modelBuilder.Entity<Model.MyERP2Model.Currency>().HasMany(e => e.NvContactBase).WithOne(e => e.NvCurrencies).HasForeignKey(e => e.CurrencyId);

                //Entity Incoterms:
                modelBuilder.Entity<Model.MyERP2Model.Incoterm>().HasMany(e => e.NvContactBase).WithOne(e => e.NvIncoterms).HasForeignKey(e => e.IncotermsId);

                //Entity Language
                modelBuilder.Entity<Model.MyERP2Model.Language>().HasMany(e => e.NvContactBase).WithOne(e => e.NvLanguage).HasForeignKey(e => e.LanguageId);
                modelBuilder.Entity<Model.MyERP2Model.Language>().HasMany(e => e.NvDescription).WithOne(e => e.NvLanguage).HasForeignKey(e => e.LanguageId);
                modelBuilder.Entity<Model.MyERP2Model.Language>().HasMany(e => e.NVTextDescriptions).WithOne(e => e.NvLanguage).HasForeignKey(e => e.LanguageId);

                //Entity Legal Form
                modelBuilder.Entity<Model.MyERP2Model.LegalForm>().HasMany(e => e.NvContactBase).WithOne(e => e.NvLegalForms).HasForeignKey(e => e.LegalformId);

                //Entity Unit
                modelBuilder.Entity<Model.MyERP2Model.Unit>().HasMany(e => e.NvAreaUnitArticle).WithOne(e => e.NvUnitArea).HasForeignKey(e => e.DimAreaUnitId);
                modelBuilder.Entity<Model.MyERP2Model.Unit>().HasMany(e => e.NvDimensionUnitArticle).WithOne(e => e.NvUnitDimension).HasForeignKey(e => e.DimDimUnitId);
                modelBuilder.Entity<Model.MyERP2Model.Unit>().HasMany(e => e.NvVolumeUnitArticle).WithOne(e => e.NvUnitVolume).HasForeignKey(e => e.DimVolumeUnitId);
                modelBuilder.Entity<Model.MyERP2Model.Unit>().HasMany(e => e.NvWeightUnitArticle).WithOne(e => e.NvUnitWeight).HasForeignKey(e => e.DimWeightUnitId);
                modelBuilder.Entity<Model.MyERP2Model.Unit>().HasMany(e => e.NvBaseUnitArticle).WithOne(e => e.NvUnitBase).HasForeignKey(e => e.BasicUnitId);
                modelBuilder.Entity<Model.MyERP2Model.Unit>().HasMany(e => e.NvSalesUnitArticle).WithOne(e => e.NvUnitSales).HasForeignKey(e => e.SalesUnitId);

                //Entity User / UserGroups / and Joining UseUserGroup
                modelBuilder.Entity<Model.MyERP2Model.UserUserGroup>().HasKey(e => new { e.UserId, e.UserGroupId });
                modelBuilder.Entity<Model.MyERP2Model.UserUserGroup>().HasOne(e => e.NvUser).WithMany(e => e.NvUserUserGroup).HasForeignKey(e => e.UserId);
                modelBuilder.Entity<Model.MyERP2Model.UserUserGroup>().HasOne(e => e.NvUserGroup).WithMany(e => e.NvUserUsergroup).HasForeignKey(e => e.UserGroupId);
                modelBuilder.Entity<Model.MyERP2Model.User>().HasMany(e => e.NvUsersSettingsGrid).WithOne(e => e.NvUser).HasForeignKey(e => e.UserId);
                modelBuilder.Entity<Model.MyERP2Model.UserSettingsGrid>().HasMany(e => e.NVUserSettingsGridColumns).WithOne(e => e.NvUserSettingsGrid).HasForeignKey(e => e.UserSettingsGridId);
                modelBuilder.Entity<Model.MyERP2Model.UserGroup>().HasMany(e => e.NvUserGroupRights).WithOne(e => e.NvUserGroup).HasForeignKey(e => e.GroupId);

                //VAT Group
                modelBuilder.Entity<Model.MyERP2Model.VatGroup>().HasMany(e => e.NvArticles).WithOne(e => e.NVVATGroup).HasForeignKey(e => e.VATGroupId);

                //DiscountGroup
                modelBuilder.Entity<Model.MyERP2Model.DiscountGroup>().HasMany(e => e.NvArticles).WithOne(e => e.NVDiscountGroup).HasForeignKey(e => e.DiscountGroupId);

                //TextDescriptionHeader
                modelBuilder.Entity<Model.MyERP2Model.TextDescriptionHeader>().HasMany(e => e.NvDescriptions).WithOne(e => e.NvHeader).HasForeignKey(e => e.HeaderId);

                //Filterdefinition
                modelBuilder.Entity<Model.MyERP2Model.FilterDefinition>().HasMany(e => e.NVFilterItems).WithOne(e => e.NVFilterDefinition).HasForeignKey(e => e.FilterId);


            //Seeding
            //Add User Administrator
            Guid UserId = Guid.NewGuid();
            modelBuilder.Entity<Model.MyERP2Model.User>().HasData(new Model.MyERP2Model.User {Id=UserId, Username = "Administrator", CreatedBy = "Seed", CreationTimestamp = DateTime.Now });
            //Add User Group Administrators
            Guid GroupId = Guid.NewGuid();
            modelBuilder.Entity<Model.MyERP2Model.UserGroup>().HasData(new Model.MyERP2Model.UserGroup {Id=GroupId, GroupName = "Administratoren", CreatedBy = "Seed", CreationTimestamp = DateTime.Now});
            //modelBuilder.Entity<Model.MyERP2Model.UserGroup>().HasData(new Model.MyERP2Model.UserGroup {GroupName = "Administratoren", CreatedBy = "Seed", CreationTimestamp = DateTime.Now });
            //Add Right to Access UserGroupRight Management
            Guid AccessRightId = Guid.NewGuid();
            Guid AddRightId = Guid.NewGuid();
            Guid EditRightId = Guid.NewGuid();
            Guid DeleteRightId = Guid.NewGuid();
            modelBuilder.Entity<Model.MyERP2Model.UserGroupRight>().HasData(new Model.MyERP2Model.UserGroupRight { Id = AccessRightId, GroupId=GroupId,  Authorization= "access_usergroups", CreatedBy = "Seed", CreationTimestamp = DateTime.Now });
            modelBuilder.Entity<Model.MyERP2Model.UserGroupRight>().HasData(new Model.MyERP2Model.UserGroupRight { Id = AddRightId, GroupId = GroupId, Authorization = "add_usergroups", CreatedBy = "Seed", CreationTimestamp = DateTime.Now });
            modelBuilder.Entity<Model.MyERP2Model.UserGroupRight>().HasData(new Model.MyERP2Model.UserGroupRight { Id = EditRightId, GroupId = GroupId, Authorization = "edit_usergroups", CreatedBy = "Seed", CreationTimestamp = DateTime.Now });
            modelBuilder.Entity<Model.MyERP2Model.UserGroupRight>().HasData(new Model.MyERP2Model.UserGroupRight { Id = DeleteRightId, GroupId = GroupId, Authorization = "delete_usergroups", CreatedBy = "Seed", CreationTimestamp = DateTime.Now });
            //Link between User and UserGroup:
            modelBuilder.Entity<Model.MyERP2Model.UserUserGroup>().HasData(new Model.MyERP2Model.UserUserGroup { UserId = UserId, UserGroupId = GroupId });
        }

        public override int SaveChanges()
        {
            UpdateEntitysBeforeSaving();
            return base.SaveChanges();
        }

        public override Task<int> SaveChangesAsync(bool acceptAllChangesOnSuccess, CancellationToken cancellationToken = default(CancellationToken))
        {
            UpdateEntitysBeforeSaving();
            return base.SaveChangesAsync(acceptAllChangesOnSuccess, cancellationToken);
        }

        /// <summary>
        /// Detaches all of the EntityEntry objects that have been added to the ChangeTracker.
        /// </summary>
        public void DetachAll()
        {
            foreach (EntityEntry entityEntry in ChangeTracker.Entries().ToArray())
            {
                if (entityEntry.Entity != null)
                {
                    entityEntry.State = EntityState.Detached;
                }
            }
        }



        private void UpdateEntitysBeforeSaving()
        {
            var objectStateEntries = ChangeTracker.Entries()
           .Where(e => e.Entity is ModelBase && e.State != EntityState.Detached && e.State != EntityState.Unchanged).ToList();

            foreach (var entry in objectStateEntries)
            {
                if (!(entry.Entity is ModelBase entityBase)) continue;
                switch (entry.State)
                {
                    case EntityState.Deleted:
                        if (entityBase is ILogicalDelete delete)
                        {
                            entry.State = EntityState.Modified;

                            delete.DeletedTimestamp = DateTime.Now;
                            delete.DeletedFlag = true;
                            entry.Property(nameof(delete.DeletedTimestamp)).IsModified = true;
                            entry.Property(nameof(delete.DeletedFlag)).IsModified = true;
                        }
                        break;
                    case EntityState.Modified:

                        if (entityBase is ILogicalTimestamp timestamp)
                        {
                            timestamp.LastUpdateTimestamp = DateTime.Now;
                            entry.Property(nameof(timestamp.LastUpdateTimestamp)).IsModified = true;

                        }
                        if (entityBase is ILogicalDelete deleteedited)
                        {
                            string originalvalue = entry.Property(nameof(deleteedited.DeletedFlag)).OriginalValue.ToString();
                            string currentValue = entry.Property(nameof(deleteedited.DeletedFlag)).CurrentValue.ToString();
                            bool.TryParse(originalvalue, out var result);
                            bool.TryParse(currentValue, out var result1);
                            if (result != result1)
                            {
                                deleteedited.DeletedTimestamp = DateTime.Now;
                                entry.Property(nameof(deleteedited.DeletedTimestamp)).IsModified = true;
                            }
                        }
                        break;
                    case EntityState.Added:
                        if (entityBase is ILogicalTimestamp logicalTimestamp)
                        {
                            logicalTimestamp.CreationTimestamp = DateTime.Now;
                            logicalTimestamp.LastUpdateTimestamp = DateTime.Now;
                            entry.Property(nameof(logicalTimestamp.CreationTimestamp)).IsModified = true;
                            entry.Property(nameof(logicalTimestamp.LastUpdateTimestamp)).IsModified = true;
                        }
                        break;
                }

            }

        }

    }
}
