﻿
Imports System.Data.SqlClient
Imports Model.MyERP2Model
Imports MyERP2.DAL

Public Class LoginBl
    Inherits BlBase

    Private ReadOnly _repUser As GenericRepository(Of User)

    Public Sub New()
        _repUser = New GenericRepository(Of User)
    End Sub


    ''' <summary>
    ''' Bei diesem Konstruktor kann eine UnitOfWork mit hereingereicht werden damit diese verwendet wird - somit kann der selbe DBContext verwendet werden!!!
    ''' </summary>
    Friend Sub New(rep As GenericRepository(Of User))
        _repUser = rep
    End Sub

    Public Async Function CanUserLogon(username As String, password As String) As Task(Of Boolean)
        Return Await _repUser.AnyAsync(_repUser.GetAll.Where(Function(x) x.Username.ToLower = username))
    End Function



    ''' <summary>
    ''' Function creates a connectionstring with the given parameters
    ''' Note: all parameters are mandatory
    ''' Created ConnectionString is stored in the singleton CurrentSessionsSettingsCls Class in Property ConnectionString
    ''' </summary>
    ''' <param name="vServer">Name of the database server</param>
    ''' <param name="vDatabase">Name of the database on the server</param>
    ''' <param name="vUsername">SQL Username to connect</param>
    ''' <param name="vPwd">Password of the user</param>
    ''' <returns>True => Connectionstring was successfully build False => something went wrong</returns>
    Public Function GetConnectionString(vServer As String, vDatabase As String, vUsername As String, vPwd As String) As String
        If String.IsNullOrEmpty(vServer) Or String.IsNullOrEmpty(vDatabase) Or String.IsNullOrEmpty(vUsername) Or String.IsNullOrEmpty(vPwd) Then Throw New ArgumentNullException()

        Dim vBuilder As SqlConnectionStringBuilder = New SqlConnectionStringBuilder() With {
                .DataSource = vServer,
                .InitialCatalog = vDatabase,
                .UserID = vUsername,
                .Password = vPwd}
        Console.WriteLine("CurrentConnectionString: {0}", vBuilder.ConnectionString)
        Return vBuilder.ConnectionString

    End Function






    Public Function GetUserInformationIncludingGroupsAndSettings(username As String) As Model.MyERP2Model.User
        Dim query = _repUser.FindBy(Function(x) x.Username.ToLower = username.ToLower)
        query = query.Include("NvUserUserGroup.NvUserGroup")
        query = query.Include(Function(x) x.NvUsersSettingsGrid)
        Return query.Single
    End Function

    Protected Overrides Sub Dispose(disposing As Boolean)
        MyBase.Dispose(disposing)

        _repUser?.Dispose()

    End Sub


End Class
